/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ScrapBook CHM Creator.
 *
 * The Initial Developer of the Original Code is Gary Harris.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *     Gomita <gomita@xuldev.org> (the author of Scrapbook)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// TODO: behaviour for notes and bookmarks


var sbChmService = {

	entryTitle : "",
	contentDir : null,
	resource : null,
	fileHHP : null,
  oSBTree : null,
  oSBData : null,
  oSBUtils : null,
	
	exec : function(aRes, aRecursive, test)
	{
    // Get property strings.
    var strbundle = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
    this.strings = strbundle.createBundle("chrome://sbchm/locale/overlay.properties"); 

    // Support SB and SB+.
    // API changes in SB 1.4.0 broke this extension. The same changes weren't made in SB+
    if (typeof(sbTreeUI) == "object"){
      // SB 1.4.0 or later.
      this.oSBTree = sbTreeUI;
      this.oSBData = ScrapBookData;
      this.oSBUtils = ScrapBookUtils;
    }
    else if (typeof(sbTreeHandler) == "object"){
      // SB < 1.4.0 or SB+.
      this.oSBTree = sbTreeHandler;
      this.oSBData = sbDataSource;
      this.oSBUtils = sbCommonUtils;
    }
    else{
      var sbVersion = this.strings.GetStringFromName("sbVersion");
			return alert(sbVersion);
		}
		// Get selected entry.
		aRes = this.oSBTree.resource;
    if(aRes === null){
      var selectEntry = this.strings.GetStringFromName("sbVersion");
      alert(selectEntry);
      return false;
    }

		this.entryTitle = (this.oSBData.getProperty(aRes, "title"));

    var id = this.oSBData.getProperty(aRes, "id");
    this.contentDir = this.oSBUtils.getContentDir(id, true);

    // Remember the selection details.
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    prefs.setCharPref("extensions.scrapbook.chm.chmtitle", this.entryTitle);
    prefs.setCharPref("extensions.scrapbook.chm.chmfilename", this.oSBUtils.validateFileName(this.entryTitle) + ".chm");
    
    if(!this.oSBData.isContainer(aRes)){
      var noCompile = prefs.getBoolPref("extensions.scrapbook.chm.helpnocompile");
      var pathOutput = prefs.getCharPref("extensions.scrapbook.chm.outputpath");
      
      if(pathOutput == ''){
        var errorOutputPath = this.strings.GetStringFromName("errorOutputPath");
        alert(errorOutputPath);
        return false;
      }
    	if(!this.IsNewFileOrCanOverwrite(pathOutput, this.oSBUtils.validateFileName(this.entryTitle + ".chm"))){
        return false;
    	}

      // An exception returned here from handleError() aborts the script.
      try{
        this.CreateHHP(prefs, pathOutput);
        // Compile the CHM if requested.
        if(!noCompile){
         	if(!this.CompileCHM(prefs))
            return false;
          this.PostProcess(prefs);
        }
        this.ReportCompletion();
      }catch(e){
        return false;
      }
    }
    else{
      // An exception returned here from handleError() aborts the script.
      try{
        this.processFolderRecursively(aRes, prefs, true);
      }catch(e){
        return false;
      }
    }
    
    return true;
	},

	CreateHHP : function(prefs, pathOutput)
	{
    var txtContent = "";
		// Get prefs.
    var pathLog = prefs.getCharPref("extensions.scrapbook.chm.logpath");
    var navPane = prefs.getBoolPref("extensions.scrapbook.chm.navpane");
    if(navPane){
      // If the navpane isn't used, we don't need these.
      var navPaneClosed = prefs.getBoolPref("extensions.scrapbook.chm.navpaneclosed");
      var navDefaultTab = prefs.getIntPref("extensions.scrapbook.chm.navdefaulttab");
      var navTabToc = prefs.getBoolPref("extensions.scrapbook.chm.navtabtoc");
      var navTabIndex = prefs.getBoolPref("extensions.scrapbook.chm.navtabindex");
      var navTabPosition = prefs.getIntPref("extensions.scrapbook.chm.navtabposition");
    }
    var language = prefs.getCharPref("extensions.scrapbook.chm.language");

		this.fileHHP = this.contentDir.clone();
		this.fileHHP.append("sbchm.hhp");
    var filenameCHM = this.oSBUtils.validateFileName(this.entryTitle) + ".chm";
    // Use nsiLocalFile to get platform independent path.
    var fileCHM = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
    var pathCHM = "";
    if(pathOutput == ""){
      pathCHM = filenameCHM;
    }else{
      try{
        fileCHM.initWithPath(pathOutput);
        fileCHM.append(filenameCHM);
        pathCHM = fileCHM.path;
      }catch(e){
        this.HandleError(e);
      }
 		}
    // ** Options section. **
    txtContent += "[OPTIONS]\n";
    txtContent += "Compatibility=1.1 or later\n";
    txtContent += "Compiled file=" + pathCHM + "\n";
    if(navTabToc){
      txtContent += "Auto TOC=9\n";
      txtContent += "Binary TOC=Yes\n";
      txtContent += "Contents file=sbchm.hhc\n";
    }
    txtContent += "Default Window=sbchmwindow\n";
    txtContent += "Default topic=index.html\n";
    txtContent += "Display compile notes=No\n";
    txtContent += "Display compile progress=No\n";
    if(pathLog != ""){
      txtContent += "Error log file=" + pathLog + "\n";
    }
    txtContent += "Full-text search=Yes\n";
    if(navTabIndex){
      txtContent += "Index file=sbchm.hhk\n";
    }
    if(language != ""){
      txtContent += "Language=" + language + "\n";
    }else{
      txtContent += "Language=0x409 English (United States)\n";
    }
    txtContent += "Title=" + this.entryTitle + "\n\n";
    // ** Windows section. **
    txtContent += "[WINDOWS]\n";
    txtContent += "sbchmwindow=\"" + this.entryTitle + "\",";
    if(navTabToc){
      txtContent += "\"sbchm.hhc\"";
    }
    txtContent += ",";
    if(navTabIndex){
      txtContent += "\"sbchm.hhk\"";
    }
    txtContent += ",\"index.html\",\"index.html\",,,,,";
    // Nav. pane style bitfield.
    txtContent += "0x" + this.BuildNavBitField(prefs, navPane).toString(16);
    txtContent += ",,";
    // Button bitfield.
    txtContent += "0x" + this.BuildBtnBitField(prefs, navPane).toString(16);
    txtContent += ",,,,,";
    // Whether or not the navigation pane is initially closed. 1 = closed, 0 = open
    if(!navPane){
      txtContent += "1";   
    }else{
      txtContent += "0";   
    }
    txtContent += ",";
    switch(navDefaultTab)
    {
    case 0:   // TOC.
    case 1:   // Index.
    case 2:   // Search.
    case 3:   // Favorites.
      txtContent += navDefaultTab;
      break;
    default:
      break;  // No default.
    }
    // Where the navigation pane tabs should be. 0 = Top, 1 = Left, 2 = Bottom
    txtContent += "," + navTabPosition;
    txtContent += ",0\n\n";
    // ** Files section. **
    txtContent += "[FILES]\n";
    // Loop through the files in the dir and find the content files.
    var entries = this.contentDir.directoryEntries;
    while(entries.hasMoreElements())
    {
      var entry = entries.getNext();
      entry.QueryInterface(Components.interfaces.nsIFile);
      // Scrapbook always creates .html files, so grab those.
      if(entry.leafName.substr(-4) == "html"){
        txtContent += entry.leafName + "\n";
      }
    }
    // ** Infotypes section. **
    txtContent += "\n[INFOTYPES]\n";
    this.oSBUtils.writeFile(this.fileHHP, txtContent, "UTF-8");
  },
	
  BuildNavBitField : function(prefs, navPane)
  {
    if(navPane){
      var navAutoSync = prefs.getBoolPref("extensions.scrapbook.chm.navautosync");
      var navAutoShowHide = prefs.getBoolPref("extensions.scrapbook.chm.navautoshowhide");
      var navTabSearch = prefs.getBoolPref("extensions.scrapbook.chm.navtabsearch");
      var navTabAdvanced = prefs.getBoolPref("extensions.scrapbook.chm.navtabadvanced");
      var navTabFavs = prefs.getBoolPref("extensions.scrapbook.chm.navtabfavs");
    }
    var bitField = 0x00002000;                  // Default: Put current HTML title in title bar.
    if(navPane){
      bitField |= 0x00000020;                   // Use a tri-pane window.
      if(navAutoShowHide){
        bitField |= 0x00000001;                 // Automatically hide/show tri-pane window.
      }
      if(navAutoSync){
        bitField |= 0x00000100;                 // When the current topic changes automatically sync contents and index.
      }
      if(navTabSearch){
        bitField |= 0x00000400;                 // Include search tab in navigation pane.
        }
      if(navTabFavs){
        bitField |= 0x00001000;                 // Include favorites tab in navigation pane.
      }
      if(navTabSearch && navTabAdvanced){
        bitField |= 0x00020000;                 // Advanced FTS UI.
      }
    }
    return bitField;
  },
	
  BuildBtnBitField : function(prefs, navPane)
  {
    var bitField = 0x00000000;
    if(navPane){
      // If the navpane isn't used, we don't need these.
      var btnHideShow = prefs.getBoolPref("extensions.scrapbook.chm.btnhideshow");
      var btnBack = prefs.getBoolPref("extensions.scrapbook.chm.btnback");
      var btnForward = prefs.getBoolPref("extensions.scrapbook.chm.btnforward");
      var btnStop = prefs.getBoolPref("extensions.scrapbook.chm.btnstop");
      var btnRefresh = prefs.getBoolPref("extensions.scrapbook.chm.btnrefresh");
      var btnHome = prefs.getBoolPref("extensions.scrapbook.chm.btnhome");
      var btnLocate = prefs.getBoolPref("extensions.scrapbook.chm.btnlocate");
      var btnOptions = prefs.getBoolPref("extensions.scrapbook.chm.btnoptions");
      var btnPrint = prefs.getBoolPref("extensions.scrapbook.chm.btnprint");
      var btnFont = prefs.getBoolPref("extensions.scrapbook.chm.btnfont");
      var btnNext = prefs.getBoolPref("extensions.scrapbook.chm.btnnext");
      var btnPrevious = prefs.getBoolPref("extensions.scrapbook.chm.btnprevious");

      if(btnHideShow){
        bitField |= 0x00000002;
      }
      if(btnBack){
        bitField |= 0x00000004;
      }
      if(btnForward){
        bitField |= 0x00000008;
      }
      if(btnStop){
        bitField |= 0x00000010;
      }
      if(btnRefresh){
        bitField |= 0x00000020;
      }
      if(btnHome){
        bitField |= 0x00000040;
      }
      if(btnLocate){
        bitField |= 0x00000800;   // Jumps to the current topic in the contents pane.
      }
      if(btnOptions){
        bitField |= 0x00001000;
      }
      if(btnPrint){
        bitField |= 0x00002000;
      }
      if(btnFont){
        bitField |= 0x00100000;
      }
      if(btnNext){
        bitField |= 0x00200000;   // Jumps to the next topic in the contents pane. Requires Binary TOC to be on.
      }
      if(btnPrevious){
        bitField |= 0x00400000;   // Jumps to the previous topic in the contents pane. Requires Binary TOC to be on.
      }
    }
    return bitField;
  },

  PostProcess : function(prefs)
  {
    // Delete proj based on prefs.
    var bDeleteProj = prefs.getBoolPref("extensions.scrapbook.chm.helpdelproj");
    if(bDeleteProj){
      var pathLog = prefs.getCharPref("extensions.scrapbook.chm.logpath");
      var navTabToc = prefs.getBoolPref("extensions.scrapbook.chm.navtabtoc");
      var navTabIndex = prefs.getBoolPref("extensions.scrapbook.chm.navtabindex");
      try{
        if(pathLog != ""){
          var fileLog = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
          if(pathLog.lastIndexOf("\\") == -1 && pathLog.lastIndexOf("/") == -1){
            // The log entry is a file name and doesn't include a path.
            // It's therefore in the content directory.
            fileLog.initWithPath(this.contentDir.path);
            fileLog.append(pathLog);
          }else{
            // The log entry is a full path and filename.
            fileLog.initWithPath(pathLog);
          }
          fileLog.remove(false);
        }
        if(navTabToc){
          var fileToc = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
          fileToc.initWithPath(this.contentDir.path);
          fileToc.append("sbchm.hhc");
          fileToc.remove(false);
        }
        if(navTabIndex){
          var fileIndex = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
          fileIndex.initWithPath(this.contentDir.path);
          fileIndex.append("sbchm.hhk");
          fileIndex.remove(false);
        }
        this.fileHHP.remove(false);
      
      }catch(e if e.name == "NS_ERROR_FILE_ACCESS_DENIED"){
        var errorAccess = this.strings.GetStringFromName("errorAccess");
        alert(errorAccess);
      }catch(e){
        this.HandleError(e);
      }
    }
  },

 	CompileCHM : function(prefs)
	{
    var pathCompiler = prefs.getCharPref("extensions.scrapbook.chm.hhcpath");
    var args = new Array();

    if(pathCompiler == ''){
      var errorCompilerPath = this.strings.GetStringFromName("errorCompilerPath");
      alert(errorCompilerPath);
      return false;
    }
    
  	args[0] = this.fileHHP.path;
    this.execProgram(pathCompiler, args);
    return true;
  },

	ReportCompletion : function()
	{
    var alertLabel = this.strings.GetStringFromName("alertLabel");
    var alertMessage = this.strings.GetStringFromName("alertMessage");
    var alerts = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
    alerts.showAlertNotification("chrome://sbchm/content/sbchm.png", alertLabel, alertMessage, false, "", null);
  },

	processFolderRecursively : function(aRes, prefs, aRecursive)
	{
    this.oSBUtils.RDFC.Init(oSBData.data, aRes);
		var resEnum = this.oSBUtils.RDFC.GetElements();
    var noCompile = prefs.getBoolPref("extensions.scrapbook.chm.helpnocompile");
    var pathOutput = prefs.getCharPref("extensions.scrapbook.chm.outputpath");

		while ( resEnum.hasMoreElements() )
		{
			var res = resEnum.getNext();
			if ( oSBData.isContainer(res) ) {
				if ( aRecursive ){
          this.processFolderRecursively(res, aRecursive);
        }
			} else {
    		// Do some title juggling.
        this.entryTitle = (oSBData.getProperty(res, "title"));
        prefs.setCharPref("extensions.scrapbook.chm.chmtitle", this.entryTitle);
        prefs.setCharPref("extensions.scrapbook.chm.chmfilename", this.oSBUtils.validateFileName(this.entryTitle) + ".chm");
    		
        var id = oSBData.getProperty(res, "id");
        this.contentDir = this.oSBUtils.getContentDir(id, true);

      	if(!this.IsNewFileOrCanOverwrite(pathOutput, this.oSBUtils.validateFileName(this.entryTitle + ".chm"))){
          continue;
      	}

        this.CreateHHP(prefs, pathOutput);
        // Compile the CHM if requested.
        if(!noCompile){
         	this.CompileCHM(prefs);
          this.PostProcess(prefs);
        }
			}
		}
    this.ReportCompletion();
	},

 	HandleError : function(e, message)
	{
    if(!message){
      var message = "";
    }
    var errorText1 = this.strings.GetStringFromName("errorText1");
    var errorText2 = this.strings.GetStringFromName("errorText2");
    var errorText3 = this.strings.GetStringFromName("errorText3");
    txt = errorText1 + e.name + ".\n\n";
    txt += errorText2 + "\n" + e.message + "\n\n";
    txt += errorText3 + "\n\n";
    alert(txt);
    // Abort the script.
    throw "stop";
	},
	
	// Borrowed from Scrapbook and changed to run the process blocking.
  execProgram : function(aExecFilePath, args)
	{
		var execfile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		var process  = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
		try {
			execfile.initWithPath(aExecFilePath);
			if ( !execfile.exists() ) {
        var errorExist = this.strings.GetStringFromName("errorExist");
				err = new Error(errorExist + "\n" + aExecFilePath);
        err.number = 999;
        throw err;
			}
			process.init(execfile);
			process.run(true, args, args.length);
		} catch (ex) {
		  if(ex.number == 999){
        this.HandleError(ex, "");
		  } else {
        var errorExecutable = this.strings.GetStringFromName("errorExecutable");
  			this.HandleError(ex, errorExecutable + "\n" + aExecFilePath);
  		}
		}
	},

  // Check if the file already exists and, if so, whether to overwrite.
	IsNewFileOrCanOverwrite : function(pathOutput, fileName)
	{
		var fileOutput = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    fileOutput.initWithPath(pathOutput);
    fileOutput.append(fileName);
    if(fileOutput.exists()){
      var promptOverwrite1 = this.strings.GetStringFromName("promptOverwrite1");
      var promptOverwrite2 = this.strings.GetStringFromName("promptOverwrite2");
      if(!confirm(fileOutput.path + "\n" + promptOverwrite1 + "\n\n" + promptOverwrite2)){
        return false;
      }
    }
    return true;
  },
	
};

	
var OverlayCHM = {
  // Update prefs on first load.

  init : function(){
    //gets the version number.
    try {
      // Firefox 4 and later; Mozilla 2 and later
      Components.utils.import("resource://gre/modules/AddonManager.jsm");
      AddonManager.getAddonByID("{BE6F2C39-C1FD-429f-8507-2702EE759A3D}", function(addon) {
        OverlayCHM.init2(addon.version);
      });
    } catch (ex) {
      // Firefox 3.6 and before; Mozilla 1.9.2 and before
      var em = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
      var addon = em.getItemForID("{BE6F2C39-C1FD-429f-8507-2702EE759A3D}");
        OverlayCHM.init2(addon.version);
    }
  },

  init2 : function(current){
    var ver = -1;
    var firstrun = false;
    var PrefsCHM = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var oldPrefsCHM = PrefsCHM.getBranch("scrapbook.chm.");
    var newPrefsCHM = PrefsCHM.getBranch("extensions.scrapbook.chm.");

    
    try{
    	ver = newPrefsCHM.getCharPref("version");
    	firstrun = newPrefsCHM.getBoolPref("firstrun");
    }catch(e){
      //nothing
    }finally{
      if (firstrun){
        newPrefsCHM.setBoolPref("firstrun",false);
        newPrefsCHM.setCharPref("version",current);
      }		

      if (ver!=current && !firstrun){ // !firstrun ensures that this section does not get loaded if its a first run.
        newPrefsCHM.setCharPref("version",current);

        // Insert code if version is different here => upgrade
        var cnt = {};
        var list = new Array();
        list = oldPrefsCHM.getChildList("", cnt);
        
        // If old prefs exist, convert and delete them.
        if(cnt.value > 0){
          for(var i in list)
          {
            var type = oldPrefsCHM.getPrefType(list[i]);
            // Only convert prefs that have user values.
            if(oldPrefsCHM.prefHasUserValue(list[i])) {
              if(type == Components.interfaces.nsIPrefBranch.PREF_STRING){
                newPrefsCHM.setCharPref(list[i], oldPrefsCHM.getCharPref(list[i]));
              }
              if(type == Components.interfaces.nsIPrefBranch.PREF_INT){
                newPrefsCHM.setIntPref(list[i], oldPrefsCHM.getIntPref(list[i]));
              }
              if(type == Components.interfaces.nsIPrefBranch.PREF_BOOL){
                newPrefsCHM.setBoolPref(list[i], oldPrefsCHM.getBoolPref(list[i]));
              }
            }
          }
          //Delete old prefs.
          oldPrefsCHM.deleteBranch("");
          PrefsCHM.savePrefFile(null);
        }
      }
      
      // The example below loads a page by opening a new tab.
      // Useful for loading a mini tutorial
      // Open the help on first run or upgrade.
      if (firstrun || (ver != current)){
        var mainWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                               .getInterface(Components.interfaces.nsIWebNavigation)
                               .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                               .rootTreeItem
                               .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                               .getInterface(Components.interfaces.nsIDOMWindow);
        
        mainWindow.gBrowser.selectedTab = mainWindow.gBrowser.addTab("chrome://sbchm/locale/sbchm.html");
      }
    }
    window.removeEventListener("load", OverlayCHM.init, true);
  }
};

window.addEventListener("load", OverlayCHM.init, true);







